export class AsyncQueue {
    constructor() {
        this.queue = []; // Holds messages that have arrived
        this.waiters = []; // Holds resolve functions for pending promises
    }
    // Called when a new message arrives (e.g., from an event)
    push(item) {
        // If someone is waiting for a message, resolve their promise immediately.
        if (this.waiters.length > 0) {
            const resolve = this.waiters.shift();
            resolve(item);
        }
        else {
            // Otherwise, add the message to the queue.
            this.queue.push(item);
        }
    }
    // Returns a promise that resolves with the next message.
    next() {
        // If there is already a message waiting, return it immediately.
        if (this.queue.length > 0) {
            return Promise.resolve(this.queue.shift());
        }
        // Otherwise, return a promise that will resolve when a new message arrives.
        return new Promise(resolve => {
            this.waiters.push(resolve);
        });
    }
}
